﻿using System.Globalization;
using CsvHelper;
using LeedsBeerQuest.API.Data.Contexts;
using LeedsBeerQuest.API.Data.Models;

namespace LeedsBeerQuest.API.Data.Seed;

public class LeedsBeerQuestSeeder
{
    private readonly LeedsBeerQuestDbContext _dbContext;
    private readonly string _dataSourcePath;

    public LeedsBeerQuestSeeder(LeedsBeerQuestDbContext dbContext, string dataSourcePath)
    {
        _dbContext = dbContext;
        _dataSourcePath = dataSourcePath;
    }

    public void Seed()
    {
        // We don't want to seed if we already have data in the database.
        if (hasExistingData)
        {
            return;
        }


        IEnumerable<LeedsBeerQuestCSV>? lbqCSV;

        using (var reader = new StreamReader(_dataSourcePath))
        using (var csv = new CsvReader(reader, CultureInfo.InvariantCulture))
        {
            lbqCSV = csv.GetRecords<LeedsBeerQuestCSV>().ToList();
        }

        var tags = getTags(lbqCSV).ToList();
        var categories = getCategories(lbqCSV).ToList();

        _dbContext.Tags.AddRange(tags);
        _dbContext.Categories.AddRange(categories);

        var venues = new List<Venue>();

        var venueIndex = 1;
        foreach (var venue in lbqCSV)
        {
            venues.Add(new Venue
            {
                Id = venueIndex,
                Name = venue.name,
                CategoryId = categories.Single(c => c.Name == venue.category).Id,
                Url = venue.url,
                DateAttended = venue.date,
                Excerpt = venue.excerpt,
                Thumbnail = venue.thumbnail,
                Latitude = venue.lat,
                Longitude = venue.lng,
                Address = venue.address,
                Phone = venue.phone,
                Twitter = venue.twitter,
                StarsBeer = venue.stars_beer,
                StarsAtmosphere = venue.stars_atmosphere,
                StarsAmenities = venue.stars_amenities,
                StarsValue = venue.stars_value,
            });

            venueIndex++;
        }

        _dbContext.Venues.AddRange(venues);

        _dbContext.SaveChanges();
    }

    private IEnumerable<Tag> getTags(IEnumerable<LeedsBeerQuestCSV> csvData)
    {
        var tagEntries = csvData.Select(d => d.tags);
        var uniqueTags = new List<string>();

        foreach (var tagEntry in tagEntries)
        {
            var tags = tagEntry.Split(',');

            foreach (var tag in tags)
            {
                if (!uniqueTags.Contains(tag))
                {
                    uniqueTags.Add(tag);
                }
            }
        }

        var tagId = 0;
        foreach (var tag in uniqueTags)
        {
            tagId++;
            yield return new Tag
            {
                Id = tagId,
                Name = tag
            };
        }
    }
    
    private IEnumerable<Category> getCategories(IEnumerable<LeedsBeerQuestCSV> csvData)
    {
        var categories = csvData.Select(d => d.category).Distinct();
        var categoryId = 0;
        foreach (var category in categories)
        {
            categoryId++;
            yield return new Category
            {
                Id = categoryId,
                Name = category
            };
        }
    }

    private bool hasExistingData =>
        _dbContext.Venues.Any() ||
        _dbContext.Tags.Any() ||
        _dbContext.Categories.Any();

    private class LeedsBeerQuestCSV
    {
        public string name { get; set; }
        public string category { get; set; }
        public string url { get; set; }
        public DateTime date { get; set; }
        public string excerpt { get; set; }
        public string thumbnail { get; set; }
        public decimal lat { get; set; }
        public decimal lng { get; set; }
        public string address { get; set; }
        public string? phone { get; set; }
        public string? twitter { get; set; }
        public decimal stars_beer { get; set; }
        public decimal stars_atmosphere { get; set; }
        public decimal stars_amenities { get; set; }
        public decimal stars_value { get; set; }
        public string tags { get; set; }
    }
}